==> masamune_core-0.2.0.tar.gz: lib/data/component/datadocument.dart <==
part of masamune.data;

/// Document that stores [DataField] with Key
/// 
/// Users do n’t really [DataField].
/// 
/// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
/// 
/// ```
/// DataDocument document = DataDocument.fromMap(
///   "data/document", {"A": "Data is A", "B": "Data is B"}
/// );
/// document["A"] // Data is A;
/// document["C"] = "Data is C";
/// ```
class DataDocument extends Document<DataField>
  implements IDataDocument<DataField> {
  static Iterable<DataField> _convertData(
    String path,
    Map<String, dynamic> data
  ) {
    List<DataField> list = ListPool.get();
    data?.forEach(
      ( key, value ) {
        if( isEmpty(key) || value == null ) return;
        list.add(
          DataField(Paths.child(path, key), value)
        );
      }
    );
    return list;
  }
  /// Process to create a new instance.
  /// 
  /// Do not use from outside the class.
  /// 
  /// [path]: Destination path.
  /// [isTemporary]: True if the data is temporary.
  @override
  @protected
  T createCloneInstance<T extends IClonable>( String path, bool isTemporary ) => DataDocument._(
    path: path,
    isTemporary: isTemporary
  ) as T;
  /// Document that stores [DataField] with Key.
  /// 
  /// Users do n’t really [DataField].
  /// 
  /// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
  /// 
  /// ```
  /// DataDocument document = DataDocument.fromMap(
  ///   "data/document", {"A": "Data is A", "B": "Data is B"}
  /// );
  /// document["A"] // Data is A;
  /// document["C"] = "Data is C";
  /// ```
  /// 
  /// [path]: Document path. If empty, a UUID will be assigned.
  factory DataDocument(
    String path
  ){
    path = path?.trimString( Const.slash )
      ?.replaceAll( "//", "/" )
      ?.replaceAll(":/", "://")
      ?.applyTags();
    if( isEmpty(path) ){
      Log.error("The path is invalid.");
      return null;
    }
    DataDocument document = PathMap.get<DataDocument>( path );
    if( document != null ) return document;
    return DataDocument._( path:path );
  }
  /// Document that stores [DataField] with Key.
  /// 
  /// Users do n’t really [DataField].
  /// 
  /// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
  /// 
  /// ```
  /// DataDocument document = DataDocument.fromMap(
  ///   "data/document", {"A": "Data is A", "B": "Data is B"}
  /// );
  /// document["A"] // Data is A;
  /// document["C"] = "Data is C";
  /// ```
  /// 
  /// [path]: Document path. If empty, a UUID will be assigned.
  /// [jsonData]: Json string representing Map.
  factory DataDocument.fromJson(
    String path,
    String jsonData
  ){
    path = path?.trimString( Const.slash )
      ?.replaceAll( "//", "/" )
      ?.replaceAll(":/", "://")
      ?.applyTags();
    if( isEmpty(path) ){
      Log.error("The path is invalid.");
      return null;
    }
    Map<String, dynamic> data = Json.decodeAsMap(jsonData);
    if( data == null ) return DataDocument( path );
    return DataDocument.fromMap(path, data);
  }
  /// Document that stores [DataField] with Key.
  /// 
  /// Users do n’t really [DataField].
  /// 
  /// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
  /// 
  /// ```
  /// DataDocument document = DataDocument.fromMap(
  ///   "data/document", {"A": "Data is A", "B": "Data is B"}
  /// );
  /// document["A"] // Data is A;
  /// document["C"] = "Data is C";
  /// ```
  /// 
  /// [path]: Document path. If empty, a UUID will be assigned.
  /// [mapData]: Map data to be stored.
  factory DataDocument.fromMap(
    String path,
    Map<String, dynamic> mapData
  ){
    path = path?.trimString( Const.slash )
      ?.replaceAll( "//", "/" )
      ?.replaceAll(":/", "://")
      ?.applyTags();
    if( isEmpty(path) ){
      Log.error("The path is invalid.");
      return null;
    }
    DataDocument document = PathMap.get<DataDocument>( path );
    if( document != null ) {
        if( mapData != null ) document.set( _convertData( path, mapData ) );
        return document;
    }
    return DataDocument._( path:path, children:_convertData( path, mapData ) );
  }
  DataDocument._(
    {
      String path,
      Iterable<DataField> children,
      bool isTemporary = false
    }
  ) : super(
    path,
    children: children,
    isTemporary: isTemporary
  );
  

  /// Get the protocol of the path.
  @override
  String get protocol => Protocol.data;


  /// Get the data.
  /// 
  /// Do not use from external class.
  /// 
  /// When using from an external class, use getInt or getString.
  /// 
  /// [key]: Key for retrieving data.
  @protected
  dynamic operator []( String key ){
    if( !this.data.containsKey( key ) ) return null;
    return this.data[key]?.data;
  }
  /// Set the data.
  /// 
  /// [key]: Key for storing data [value]: Data to store.
  void operator []=( String key, dynamic value ){
    if( value == null ) { return; }
    if( this.isLock ) { Log.error( "Data modification is prohibited." ); return; }
    if( value is DataField ){
      this.set( [value] );
    } else {
      if( PathFilter.setTemporary( Paths.child(this.path, key), value) ) return;
      if( this.data.containsKey( key ) && this.data[key] is IUnit<dynamic> ) {
         this.data[key].data = value;
      } else {
        Path path = this.rawPath.clone( );
        path.path = Paths.child( this.rawPath.path, key );
        this.set( [DataField( path.path, value )] );
      }
    }
  }

  /// Gets value as Bool.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  bool getBool(String key, [bool defaultValue = false]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getBool( defaultValue );
  }
  /// Gets value as Double.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  double getDouble(String key, [double defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getDouble( defaultValue );
  }
  /// Gets value as Int.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  int getInt(String key, [int defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getInt( defaultValue );
  }
  /// Gets value as List<T>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  List<T> getList<T extends Object>(
    String key,
    [
      List<T> defaultValue = const []
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getList<T>( defaultValue );
  }
  /// Gets value as Map<K,V>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  Map<K, V> getMap<K extends Object, V extends Object>(
    String key,
    [
      Map<K, V> defaultValue = const {}
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getMap<K,V>( defaultValue );
  }
  /// Gets value as String.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  String getString(String key, [String defaultValue = Const.empty]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getString( defaultValue );
  }


  /// Get the UID of the document.
  /// 
  /// If there is no value in the field, id will be output.
  String get uid => this.getString(Const.uid, this.id);
  /// Get time.
  /// 
  /// UpdatedTime is output if the field has no value.
  int get time => this.getInt(Const.time, this.updatedTime);


  /// Save the data.
  /// 
  /// Run if you have a remote or need to save data.
  Future<T> save<T extends IDataDocument>( ) => Future<T>.delayed(Duration.zero);
  /// Delete the data.
  /// 
  /// Used when deleting data when there is a remote or when data needs to be saved.
  Future delete( ){
    if( this.isDisposed ) return Future.delayed(Duration.zero);
    this.dispose();
    return Future.delayed(Duration.zero);
  }
}

==> masamune_core-0.2.2.tar.gz: lib/data/component/datadocument.dart <==
part of masamune.data;

/// Document that stores [DataField] with Key
/// 
/// Users do n’t really [DataField].
/// 
/// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
/// 
/// ```
/// DataDocument document = DataDocument.fromMap(
///   "data/document", {"A": "Data is A", "B": "Data is B"}
/// );
/// document["A"] // Data is A;
/// document["C"] = "Data is C";
/// ```
class DataDocument extends Document<DataField>
  implements IDataDocument<DataField> {
  static Iterable<DataField> _convertData(
    String path,
    Map<String, dynamic> data
  ) {
    List<DataField> list = ListPool.get();
    data?.forEach(
      ( key, value ) {
        if( isEmpty(key) || value == null ) return;
        list.add(
          DataField(Paths.child(path, key), value)
        );
      }
    );
    return list;
  }
  /// Process to create a new instance.
  /// 
  /// Do not use from outside the class.
  /// 
  /// [path]: Destination path.
  /// [isTemporary]: True if the data is temporary.
  @override
  @protected
  T createInstance<T extends IClonable>( String path, bool isTemporary ) => DataDocument._(
    path: path,
    isTemporary: isTemporary
  ) as T;
  /// Document that stores [DataField] with Key.
  /// 
  /// Users do n’t really [DataField].
  /// 
  /// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
  /// 
  /// ```
  /// DataDocument document = DataDocument.fromMap(
  ///   "data/document", {"A": "Data is A", "B": "Data is B"}
  /// );
  /// document["A"] // Data is A;
  /// document["C"] = "Data is C";
  /// ```
  /// 
  /// [path]: Document path. If empty, a UUID will be assigned.
  factory DataDocument(
    String path
  ){
    path = path?.applyTags();
    if( isEmpty(path) ){
      Log.error("The path is invalid.");
      return null;
    }
    DataDocument document = PathMap.get<DataDocument>( path );
    if( document != null ) return document;
    return DataDocument._( path:path );
  }
  /// Document that stores [DataField] with Key.
  /// 
  /// Users do n’t really [DataField].
  /// 
  /// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
  /// 
  /// ```
  /// DataDocument document = DataDocument.fromMap(
  ///   "data/document", {"A": "Data is A", "B": "Data is B"}
  /// );
  /// document["A"] // Data is A;
  /// document["C"] = "Data is C";
  /// ```
  /// 
  /// [path]: Document path. If empty, a UUID will be assigned.
  /// [jsonData]: Json string representing Map.
  factory DataDocument.fromJson(
    String path,
    String jsonData
  ){
    path = path?.applyTags();
    if( isEmpty(path) ){
      Log.error("The path is invalid.");
      return null;
    }
    Map<String, dynamic> data = Json.decodeAsMap(jsonData);
    if( data == null ) return DataDocument( path );
    return DataDocument.fromMap(path, data);
  }
  /// Document that stores [DataField] with Key.
  /// 
  /// Users do n’t really [DataField].
  /// 
  /// Normal, Can be treated as if the data were handled [Map<String, dynamic>].
  /// 
  /// ```
  /// DataDocument document = DataDocument.fromMap(
  ///   "data/document", {"A": "Data is A", "B": "Data is B"}
  /// );
  /// document["A"] // Data is A;
  /// document["C"] = "Data is C";
  /// ```
  /// 
  /// [path]: Document path. If empty, a UUID will be assigned.
  /// [mapData]: Map data to be stored.
  factory DataDocument.fromMap(
    String path,
    Map<String, dynamic> mapData
  ){
    path = path?.applyTags();
    if( isEmpty(path) ){
      Log.error("The path is invalid.");
      return null;
    }
    DataDocument document = PathMap.get<DataDocument>( path );
    if( document != null ) {
        if( mapData != null ) document.set( _convertData( path, mapData ) );
        return document;
    }
    return DataDocument._( path:path, children:_convertData( path, mapData ) );
  }
  DataDocument._(
    {
      String path,
      Iterable<DataField> children,
      bool isTemporary = false
    }
  ) : super(
    path,
    children: children,
    isTemporary: isTemporary
  );
  

  /// Get the protocol of the path.
  @override
  String get protocol => Protocol.data;


  /// Get the data.
  /// 
  /// Do not use from external class.
  /// 
  /// When using from an external class, use getInt or getString.
  /// 
  /// [key]: Key for retrieving data.
  @protected
  dynamic operator []( String key ){
    if( !this.data.containsKey( key ) ) return null;
    return this.data[key]?.data;
  }
  /// Set the data.
  /// 
  /// [key]: Key for storing data [value]: Data to store.
  void operator []=( String key, dynamic value ){
    if( value == null ) { return; }
    if( this.isLock ) { Log.error( "Data modification is prohibited." ); return; }
    if( value is DataField ){
      this.set( [value] );
    } else {
      if( PathFilter.setTemporary( Paths.child(this.path, key), value) ) return;
      if( this.data.containsKey( key ) && this.data[key] is IUnit<dynamic> ) {
         this.data[key].data = value;
      } else {
        Path path = this.rawPath.clone( );
        path.path = Paths.child( this.rawPath.path, key );
        this.set( [DataField( path.path, value )] );
      }
    }
  }

  /// Gets value as Bool.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  bool getBool(String key, [bool defaultValue = false]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getBool( defaultValue );
  }
  /// Gets value as Double.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  double getDouble(String key, [double defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getDouble( defaultValue );
  }
  /// Gets value as Int.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  int getInt(String key, [int defaultValue = 0]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getInt( defaultValue );
  }
  /// Gets value as List<T>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  List<T> getList<T extends Object>(
    String key,
    [
      List<T> defaultValue = const []
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getList<T>( defaultValue );
  }
  /// Gets value as Map<K,V>.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  Map<K, V> getMap<K extends Object, V extends Object>(
    String key,
    [
      Map<K, V> defaultValue = const {}
    ]
  ) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getMap<K,V>( defaultValue );
  }
  /// Gets value as String.
  /// 
  /// [key]: Key for retrieving data.
  /// [defaultValue]: Initial value if there is no value.
  @override
  String getString(String key, [String defaultValue = Const.empty]) {
    if( isEmpty(key) || !this.data.containsKey(key) ) return defaultValue;
    DataField data = this.data[key];
    if( data == null ) return defaultValue;
    return this.data[key].getString( defaultValue );
  }


  /// Get the UID of the document.
  /// 
  /// If there is no value in the field, id will be output.
  String get uid => this.getString(Const.uid, this.id);
  /// Get time.
  /// 
  /// UpdatedTime is output if the field has no value.
  int get time => this.getInt(Const.time, this.updatedTime);


  /// Save the data.
  /// 
  /// Run if you have a remote or need to save data.
  Future<T> save<T extends IDataDocument>( ) => Future<T>.delayed(Duration.zero);
  /// Delete the data.
  /// 
  /// Used when deleting data when there is a remote or when data needs to be saved.
  Future delete( ){
    if( this.isDisposed ) return Future.delayed(Duration.zero);
    this.dispose();
    return Future.delayed(Duration.zero);
  }
}
